@@ -20,22 +20,30 @@ IS_THREAD_SAFE = True
 
 
 class Variable(KerasVariable):
+    def __init__(self, *args, layout=None, **kwargs):
+        # Intercept layout parameter so that it is available
+        # during initialization.
+        self._layout = layout
+        super().__init__(*args, **kwargs)
+
     def _initialize(self, value):
         # Note that variable.shape is needed by distribution_lib
         self._shape = self._validate_shape(value.shape)
         # We can't import the keras/distribution/distribution_lib
         # due to circular dependency.
         distribution = global_state.get_global_attribute("distribution")
-        if distribution is not None:
-            self._layout = distribution_lib._to_jax_layout(
-                distribution.get_variable_layout(self)
-            )
-        else:
-            self._layout = None
+        if self._layout is None and distribution is not None:
+            tensor_layout = distribution.get_variable_layout(self)
+            from keras.src.distribution import TensorLayout
+
+            if isinstance(tensor_layout, TensorLayout):
+                self._layout = tensor_layout.backend_layout
+            else:
+                self._layout = tensor_layout
         self._direct_assign(value)
 
     def _direct_assign(self, value):
-        if getattr(self, "_layout", None) is not None:
+        if self._layout is not None:
             value = distribution_lib.distribute_variable(value, self._layout)
         self._value = value
 
@@ -410,7 +418,7 @@ def device_scope(device_name):
     if isinstance(device_name, str):
         # We support string value like "cpu:0", "gpu:1", etc.
         device_name = device_name.lower()
-        jax_device = distribution_lib._to_jax_device(device_name)
+        jax_device = distribution_lib._to_backend_device(device_name)
     elif not isinstance(device_name, jax.Device):
         raise ValueError(
             "Invalid value for argument `device_name`. "
