@@ -41,6 +41,38 @@ OPENVINO_DTYPES = {
     "string": ov.Type.string,
 }
 
+DTYPES_MAX = {
+    ov.Type.bf16: 3.38953139e38,
+    ov.Type.f16: np.finfo(np.float16).max,
+    ov.Type.f32: np.finfo(np.float32).max,
+    ov.Type.f64: np.finfo(np.float64).max,
+    ov.Type.u8: np.iinfo(np.uint8).max,
+    ov.Type.u16: np.iinfo(np.uint16).max,
+    ov.Type.u32: np.iinfo(np.uint32).max,
+    ov.Type.u64: np.iinfo(np.uint64).max,
+    ov.Type.i8: np.iinfo(np.int8).max,
+    ov.Type.i16: np.iinfo(np.int16).max,
+    ov.Type.i32: np.iinfo(np.int32).max,
+    ov.Type.i64: np.iinfo(np.int64).max,
+    ov.Type.boolean: 1,
+}
+
+DTYPES_MIN = {
+    ov.Type.bf16: -3.38953139e38,
+    ov.Type.f16: np.finfo(np.float16).min,
+    ov.Type.f32: np.finfo(np.float32).min,
+    ov.Type.f64: np.finfo(np.float64).min,
+    ov.Type.u8: np.iinfo(np.uint8).min,
+    ov.Type.u16: np.iinfo(np.uint16).min,
+    ov.Type.u32: np.iinfo(np.uint32).min,
+    ov.Type.u64: np.iinfo(np.uint64).min,
+    ov.Type.i8: np.iinfo(np.int8).min,
+    ov.Type.i16: np.iinfo(np.int16).min,
+    ov.Type.i32: np.iinfo(np.int32).min,
+    ov.Type.i64: np.iinfo(np.int64).min,
+    ov.Type.boolean: 0,
+}
+
 
 def align_operand_types(x1, x2, op_name):
     x1_type = x1.element_type
@@ -73,7 +105,10 @@ def get_ov_output(x, ov_type=None):
             ov_type = Type.i32
         x = ov_opset.constant(x, ov_type).output(0)
     elif isinstance(x, np.ndarray):
-        x = ov_opset.constant(x).output(0)
+        if x.dtype == np.dtype("bfloat16"):
+            x = ov_opset.constant(x, OPENVINO_DTYPES["bfloat16"]).output(0)
+        else:
+            x = ov_opset.constant(x).output(0)
     elif np.isscalar(x):
         x = ov_opset.constant(x).output(0)
     elif isinstance(x, KerasVariable):
@@ -415,8 +450,13 @@ def convert_to_tensor(x, dtype=None, sparse=None, ragged=None):
 def convert_to_numpy(x):
     if isinstance(x, np.ndarray):
         return x
-    elif isinstance(x, (int, float, list, tuple)):
+    elif isinstance(x, (int, float)):
         return np.array(x)
+    elif isinstance(x, (list, tuple)):
+        x_new = []
+        for elem in x:
+            x_new.append(convert_to_numpy(elem))
+        return np.array(x_new)
     elif np.isscalar(x):
         return x
     elif isinstance(x, ov.Tensor):
