@@ -74,12 +74,13 @@ class ModelCheckpoint(Callback):
             which will be filled the value of `epoch` and keys in `logs`
             (passed in `on_epoch_end`).
             The `filepath` name needs to end with `".weights.h5"` when
-            `save_weights_only=True` or should end with `".keras"` when
-            checkpoint saving the whole model (default).
+            `save_weights_only=True` or should end with `".keras"` or `".h5"`
+            when checkpoint saving the whole model (default).
             For example:
-            if `filepath` is `"{epoch:02d}-{val_loss:.2f}.keras"`, then the
-            model checkpoints will be saved with the epoch number and the
-            validation loss in the filename. The directory of the filepath
+            if `filepath` is `"{epoch:02d}-{val_loss:.2f}.keras"` or
+            "{epoch:02d}-{val_loss:.2f}.weights.h5"`, then the model
+            checkpoints will be saved with the epoch number and the validation
+            loss in the filename. The directory of the filepath
             should not be reused by any other callbacks to avoid conflicts.
         monitor: The metric name to monitor. Typically the metrics are set by
             the `Model.compile` method. Note:
@@ -157,20 +158,20 @@ class ModelCheckpoint(Callback):
         if mode == "min":
             self.monitor_op = np.less
             if self.best is None:
-                self.best = np.Inf
+                self.best = np.inf
         elif mode == "max":
             self.monitor_op = np.greater
             if self.best is None:
-                self.best = -np.Inf
+                self.best = -np.inf
         else:
             if "acc" in self.monitor or self.monitor.startswith("fmeasure"):
                 self.monitor_op = np.greater
                 if self.best is None:
-                    self.best = -np.Inf
+                    self.best = -np.inf
             else:
                 self.monitor_op = np.less
                 if self.best is None:
-                    self.best = np.Inf
+                    self.best = np.inf
 
         if self.save_freq != "epoch" and not isinstance(self.save_freq, int):
             raise ValueError(
@@ -187,7 +188,9 @@ class ModelCheckpoint(Callback):
                     f"filepath={self.filepath}"
                 )
         else:
-            if not self.filepath.endswith(".keras"):
+            if not any(
+                self.filepath.endswith(ext) for ext in (".keras", ".h5")
+            ):
                 raise ValueError(
                     "The filepath provided must end in `.keras` "
                     "(Keras model format). Received: "
@@ -221,6 +224,69 @@ class ModelCheckpoint(Callback):
             return True
         return False
 
+    def _should_save_model(self, epoch, batch, logs, filepath):
+        """Determines whether the model should be saved.
+
+        The model should be saved in the following cases:
+
+        - self.save_best_only is False
+        - self.save_best_only is True and `monitor` is a numpy array or
+          backend tensor (falls back to `save_best_only=False`)
+        - self.save_best_only is True and `self.monitor_op(current, self.best)`
+          evaluates to True.
+
+        Args:
+            epoch: the epoch this iteration is in.
+            batch: the batch this iteration is in. `None` if the `save_freq`
+                is set to `"epoch"`.
+            logs: the `logs` dict passed in to `on_batch_end` or
+                `on_epoch_end`.
+            filepath: the path where the model would be saved
+        """
+        logs = logs or {}
+        if self.save_best_only:
+            current = logs.get(self.monitor)
+            if current is None:
+                warnings.warn(
+                    f"Can save best model only with {self.monitor} available.",
+                    stacklevel=2,
+                )
+                return True
+            elif (
+                isinstance(current, np.ndarray) or backend.is_tensor(current)
+            ) and len(current.shape) > 0:
+                warnings.warn(
+                    "Can save best model only when `monitor` is "
+                    f"a scalar value. Received: {current}. "
+                    "Falling back to `save_best_only=False`."
+                )
+                return True
+            else:
+                if self.monitor_op(current, self.best):
+                    if self.verbose > 0:
+                        io_utils.print_msg(
+                            f"\nEpoch {epoch + 1}: {self.monitor} "
+                            "improved "
+                            f"from {self.best:.5f} to {current:.5f}, "
+                            f"saving model to {filepath}"
+                        )
+                    self.best = current
+                    return True
+                else:
+                    if self.verbose > 0:
+                        io_utils.print_msg(
+                            f"\nEpoch {epoch + 1}: "
+                            f"{self.monitor} did not improve "
+                            f"from {self.best:.5f}"
+                        )
+                    return False
+        else:
+            if self.verbose > 0:
+                io_utils.print_msg(
+                    f"\nEpoch {epoch + 1}: saving model to {filepath}"
+                )
+            return True
+
     def _save_model(self, epoch, batch, logs):
         """Saves the model.
 
@@ -230,59 +296,15 @@ class ModelCheckpoint(Callback):
                 is set to `"epoch"`.
             logs: the `logs` dict passed in to `on_batch_end` or `on_epoch_end`.
         """
-        logs = logs or {}
-
         filepath = self._get_file_path(epoch, batch, logs)
-        # Create host directory if it doesn't exist.
-        dirname = os.path.dirname(filepath)
-        if dirname and not file_utils.exists(dirname):
-            file_utils.makedirs(dirname)
 
         try:
-            if self.save_best_only:
-                current = logs.get(self.monitor)
-                if current is None:
-                    warnings.warn(
-                        f"Can save best model only with {self.monitor} "
-                        "available, skipping.",
-                        stacklevel=2,
-                    )
-                elif (
-                    isinstance(current, np.ndarray)
-                    or backend.is_tensor(current)
-                ) and len(current.shape) > 0:
-                    warnings.warn(
-                        "Can save best model only when `monitor` is "
-                        f"a scalar value. Received: {current}. "
-                        "Falling back to `save_best_only=False`."
-                    )
-                    self.model.save(filepath, overwrite=True)
-                else:
-                    if self.monitor_op(current, self.best):
-                        if self.verbose > 0:
-                            io_utils.print_msg(
-                                f"\nEpoch {epoch + 1}: {self.monitor} "
-                                "improved "
-                                f"from {self.best:.5f} to {current:.5f}, "
-                                f"saving model to {filepath}"
-                            )
-                        self.best = current
-                        if self.save_weights_only:
-                            self.model.save_weights(filepath, overwrite=True)
-                        else:
-                            self.model.save(filepath, overwrite=True)
-                    else:
-                        if self.verbose > 0:
-                            io_utils.print_msg(
-                                f"\nEpoch {epoch + 1}: "
-                                f"{self.monitor} did not improve "
-                                f"from {self.best:.5f}"
-                            )
-            else:
-                if self.verbose > 0:
-                    io_utils.print_msg(
-                        f"\nEpoch {epoch + 1}: saving model to {filepath}"
-                    )
+            if self._should_save_model(epoch, batch, logs, filepath):
+                # Create host directory if it doesn't exist.
+                dirname = os.path.dirname(filepath)
+                if dirname and not file_utils.exists(dirname):
+                    file_utils.makedirs(dirname)
+
                 if self.save_weights_only:
                     self.model.save_weights(filepath, overwrite=True)
                 else:
@@ -341,7 +363,7 @@ class ModelCheckpoint(Callback):
         later time of modification (for instance, when epoch/batch is used as
         formatting option), but not necessarily (when accuracy or loss is used).
         The tie-breaker is put in the logic as best effort to return the most
-        recent, and to avoid undeterministic result.
+        recent, and to avoid nondeterministic result.
 
         Modified time of a file is obtained with `os.path.getmtime()`.
 
