@@ -25,6 +25,8 @@ class TorchModuleWrapper(Layer):
             instance, then its parameters must be initialized before
             passing the instance to `TorchModuleWrapper` (e.g. by calling
             it once).
+        output_shape :The shape of the output of this layer. It helps Keras
+            perform automatic shape inference.
         name: The name of the layer (string).
 
     Example:
@@ -80,7 +82,7 @@ class TorchModuleWrapper(Layer):
     ```
     """
 
-    def __init__(self, module, name=None, **kwargs):
+    def __init__(self, module, name=None, output_shape=None, **kwargs):
         super().__init__(name=name, **kwargs)
         import torch.nn as nn
 
@@ -98,6 +100,7 @@ class TorchModuleWrapper(Layer):
 
         self.module = module.to(get_device())
         self._track_module_parameters()
+        self.output_shape = output_shape
 
     def parameters(self, recurse=True):
         return self.module.parameters(recurse=recurse)
@@ -138,13 +141,21 @@ class TorchModuleWrapper(Layer):
             state_dict[key] = convert_to_tensor(store[key])
         self.module.load_state_dict(state_dict)
 
+    def compute_output_shape(self, input_shape):
+        if self.output_shape is None:
+            return super().compute_output_shape(input_shape)
+        return self.output_shape
+
     def get_config(self):
         base_config = super().get_config()
         import torch
 
         buffer = io.BytesIO()
         torch.save(self.module, buffer)
-        config = {"module": buffer.getvalue()}
+        config = {
+            "module": buffer.getvalue(),
+            "output_shape": self.output_shape,
+        }
         return {**base_config, **config}
 
     @classmethod
